@@ -130,44 +130,12 @@ def repurpose(X_repurpose, model, drug_names = None,
 					print('checkout ' + fo + ' for the whole list')
 					break
 	return y_pred
-  
-# def mpnn_feature_collate_func(x): 
-# 	## first version 
-# 	return [torch.cat([x[j][i] for j in range(len(x))], 0) for i in range(len(x[0]))]
-
-
-# def mpnn_feature_collate_func(x):
-# 	assert len(x[0]) == 5
-# 	N_atoms_N_bonds = [i[-1] for i in x]
-# 	N_atoms_scope = []
-# 	f_a = torch.cat([x[j][0] for j in range(len(x))], 0)
-# 	f_b = torch.cat([x[j][1] for j in range(len(x))], 0)
-# 	agraph_lst, bgraph_lst = [], []
-# 	Na, Nb = 0, 0
-# 	for j in range(len(x)):
-# 		agraph_lst.append(x[j][2] + Na)
-# 		bgraph_lst.append(x[j][3] + Nb)
-# 		N_atoms_scope.append([Na, x[j][2].shape[0]])
-# 		Na += x[j][2].shape[0]
-# 		Nb += x[j][3].shape[0]
-# 	agraph = torch.cat(agraph_lst, 0)
-# 	bgraph = torch.cat(bgraph_lst, 0)
-# 	return [f_a, f_b, agraph, bgraph, N_atoms_scope]
-
-# def mpnn_collate_func(x):
-# 	#print("len(x) is ", len(x)) ## batch_size == 128 
-# 	#print("len(x[0]) is ", len(x[0])) ## 3--- data_process_loader_Property_Prediction.__getitem__ 
-# 	#print("len(x[1]) is ", len(x[1])) ## 3--- data_process_loader_Property_Prediction.__getitem__ 
-# 	#print("len(x[2]) is ", len(x[2])) ## 3--- data_process_loader_Property_Prediction.__getitem__ 
-
-# 	mpnn_feature = [i[0] for i in x]
-# 	#print("len(mpnn_feature)", len(mpnn_feature), "len(mpnn_feature[0])", len(mpnn_feature[0]))
-# 	mpnn_feature = mpnn_feature_collate_func(mpnn_feature)
-# 	from torch.utils.data.dataloader import default_collate
-# 	x_remain = [[i[1]] for i in x]
-# 	x_remain_collated = default_collate(x_remain)
-# 	return [mpnn_feature] + x_remain_collated
-# ## used in dataloader 
+ 
+def dgl_collate_func(x):
+	x, y = zip(*x)
+	import dgl
+	x = dgl.batch(x)
+	return x, torch.tensor(y)
 
 class Property_Prediction:
 	'''
@@ -188,6 +156,31 @@ class Property_Prediction:
 			self.model_drug = transformer('drug', **config)
 		elif drug_encoding == 'MPNN':
 			self.model_drug = MPNN(config['hidden_dim_drug'], config['mpnn_depth'])
+		elif drug_encoding == 'DGL_GCN':
+			self.model_drug = DGL_GCN(in_feats = 74, 
+									hidden_feats = [config['gnn_hid_dim_drug']] * config['gnn_num_layers'], 
+									activation = [config['gnn_activation']] * config['gnn_num_layers'], 
+									predictor_dim = config['hidden_dim_drug'])
+		elif drug_encoding == 'DGL_NeuralFP':
+			self.model_drug = DGL_NeuralFP(in_feats = 74, 
+									hidden_feats = [config['gnn_hid_dim_drug']] * config['gnn_num_layers'], 
+									max_degree = config['neuralfp_max_degree'],
+									activation = [config['gnn_activation']] * config['gnn_num_layers'], 
+									predictor_hidden_size = config['neuralfp_predictor_hid_dim'],
+									predictor_dim = config['hidden_dim_drug'],
+									predictor_activation = config['neuralfp_predictor_activation'])
+		elif drug_encoding == 'DGL_GIN_AttrMasking':
+			self.model_drug = DGL_GIN_AttrMasking(predictor_dim = config['hidden_dim_drug'])
+		elif drug_encoding == 'DGL_GIN_ContextPred':
+			self.model_drug = DGL_GIN_ContextPred(predictor_dim = config['hidden_dim_drug'])
+		elif drug_encoding == 'DGL_AttentiveFP':
+			self.model_drug = DGL_AttentiveFP(node_feat_size = 39, 
+											edge_feat_size = 11,  
+											num_layers = config['gnn_num_layers'], 
+											num_timesteps = config['attentivefp_num_timesteps'], 
+											graph_feat_size = config['gnn_hid_dim_drug'], 
+											predictor_dim = config['hidden_dim_drug'])
+
 		else:
 			raise AttributeError('Please use one of the available encoding method.')
 
@@ -210,7 +203,7 @@ class Property_Prediction:
 		y_label = []
 		model.eval()
 		for i, (v_d, label) in enumerate(data_generator):
-			if self.drug_encoding == "MPNN" or self.drug_encoding == 'Transformer':
+			if self.drug_encoding in ["MPNN", 'Transformer', 'DGL_GCN', 'DGL_NeuralFP', 'DGL_GIN_AttrMasking', 'DGL_GIN_ContextPred', 'DGL_AttentiveFP']:
 				v_d = v_d
 			else:
 				v_d = v_d.float().to(self.device)                
@@ -292,6 +285,8 @@ class Property_Prediction:
 	    		'drop_last': False}
 		if (self.drug_encoding == "MPNN"):
 			params['collate_fn'] = mpnn_collate_func
+		elif self.drug_encoding in ['DGL_GCN', 'DGL_NeuralFP', 'DGL_GIN_AttrMasking', 'DGL_GIN_ContextPred', 'DGL_AttentiveFP']:
+			params['collate_fn'] = dgl_collate_func
 
 		training_generator = data.DataLoader(data_process_loader_Property_Prediction(train.index.values, 
 																					 train.Label.values, 
@@ -312,6 +307,8 @@ class Property_Prediction:
         
 			if (self.drug_encoding == "MPNN"):
 				params_test['collate_fn'] = mpnn_collate_func
+			elif self.drug_encoding in ['DGL_GCN', 'DGL_GAT', 'DGL_NeuralFP', 'DGL_GIN_AttrMasking', 'DGL_GIN_ContextPred', 'DGL_AttentiveFP']:
+				params_test['collate_fn'] = dgl_collate_func
 			testing_generator = data.DataLoader(data_process_loader_Property_Prediction(test.index.values, test.Label.values, test, **self.config), **params_test)
 
 		# early stopping
@@ -336,7 +333,7 @@ class Property_Prediction:
 		for epo in range(train_epoch):
 			for i, (v_d, label) in enumerate(training_generator):
 				
-				if self.drug_encoding == "MPNN" or self.drug_encoding == 'Transformer':
+				if self.drug_encoding in ["MPNN", 'Transformer', 'DGL_GCN', 'DGL_NeuralFP', 'DGL_GIN_AttrMasking', 'DGL_GIN_ContextPred', 'DGL_AttentiveFP']:
 					v_d = v_d
 				else:
 					v_d = v_d.float().to(self.device)                
@@ -465,7 +462,8 @@ class Property_Prediction:
 
 		if (self.drug_encoding == "MPNN"):
 			params['collate_fn'] = mpnn_collate_func
-
+		elif self.drug_encoding in ['DGL_GCN', 'DGL_NeuralFP', 'DGL_GIN_AttrMasking', 'DGL_GIN_ContextPred', 'DGL_AttentiveFP']:
+			params['collate_fn'] = dgl_collate_func
 
 		generator = data.DataLoader(info, **params)
 
